REDUCE_PROMPT = """
You are a copyeditor. Combine the below summaries. The combined output must be less than 4,000 characters long. 
You must keep the content and context preserved. 

Input: {text} 

Output:
"""